      Module ModXpr
      Use ModOpr
      Use ModUVE
      Use ModGam
      Use ModExp
      Use ModBrk
      Implicit None
      Integer NumPow,NumExp
      Parameter (NumPow=8)
      Parameter (NumExp=1500)

      Type Expression
         Integer, Dimension(NumPow)           ::  IPowr
C        IPowr(1) ... factor numerator
C        IPowr(2) ... factor denumerator
C        IPowr(3) ... power of i
C        IPowr(4) ... power 0f e
C        IPowr(5) ... power of pi numerator
C        IPowr(6) ... power of pi denumerator
C        IPowr(7) ... divergence index: double of power of the
C                     integration variable in denominator
C        IPowr(8) ...
         Type(Operator)                       ::  Oper     
         Type(UVFunction)                     ::  UVE
         Type(Gamma)                          ::  Gam
         Type(Bracket)                        ::  Denominator
         Type(ExpFunction)                    ::  Exponent
C        Type(MomExp)                         ::  Delta
      End Type Expression

      Interface Operator (.grth.)
         Module Procedure GrTh1
      End Interface      
      Interface Operator (*)
         Module Procedure JoinAl
      End Interface

      Contains
C############################################################
      Function GrTh1(A,B)
      Implicit None
      Type(Expression),Intent(In)      :: A,B
      Logical               :: GrTh1
      Integer IPrint
      COMMON /Print/ IPrint

      GrTh1 = .false.

      If(A%Oper .grth. B%Oper) then
         GrTh1 = .true.
         Return
      End If
      If(B%Oper .grth . A%Oper) Return

C     We come here if A%Oper = B%Oper
      If(A%UVE .grth. B%UVE) then
         GrTh1 = .true.
         Return
      End If
      If(B%UVE .grth . A%UVE) Return

C     We come here if A%UVE = B%UVE
      If(A%Gam .grth. B%Gam) then
         GrTh1 = .true.
         Return
      End If
      If(B%Gam .grth . A%Gam) Return

C     We come here if A%Gam = B%Gam
      If(A%Denominator .grth. B%Denominator) then
         GrTh1 = .true.
         Return
      End If
      If(B%Denominator .grth. A%Denominator) Return

C     We come here if A%Denominator = B%Denominator
C      If(A%Delta .grth. B%Delta) then
C         GrTh1 = .true.
C         Return
C      End If
C      If(B%Delta .grth. A%Delta) Return

C     We come here if A%Delta = B%Delta
      If(A%Exponent .grth. B%Exponent) GrTh1 = .true.
      End Function GrTh1
C############################################################
      Function JoinAl(Exp1,Exp2)
      Implicit None
      Type(Expression),Intent(In)        :: Exp1, Exp2
      Type(Expression)                   :: JoinAl
      Integer IPrint,ILast
      COMMON /Print/ IPrint

      If(IPrint.ge.10)  Call Output(6,Exp1,'Term1',-1)
      If(IPrint.ge.10)  Call Output(6,Exp2,'Term2',-1)
C     Find numerical multiplier for the third expression
      JoinAl%IPowr(2) = Exp1%IPowr(2)*Exp2%IPowr(2)
      JoinAl%IPowr(1) = Exp1%IPowr(1)*Exp2%IPowr(1)
C     Find power of i and power of e
      JoinAl%IPowr(3) = Exp1%IPowr(3)+Exp2%IPowr(3)
      JoinAl%IPowr(4) = Exp1%IPowr(4)+Exp2%IPowr(4)
C     Find power of 2pi
      JoinAl%IPowr(6) = Exp1%IPowr(6)*Exp2%IPowr(6)
      JoinAl%IPowr(5) = Exp1%IPowr(5)*Exp2%IPowr(6)
     $              + Exp2%IPowr(5)*Exp1%IPowr(6)
      JoinAl%IPowr(7) = 0

C     Rewrite momentum variables from p* to p$.
      Call SubstituteLabelsExpR(Exp2,'*','$')
      Call SubstituteLabelsGam(Exp2%Gam,'*','$')

C     Rearrange spinor indices

      ILast=0
      Call Relabl(ILast,Exp1%UVE,Exp1%Gam,Exp1%Oper)
C     ILast returns largest label used for term '1'
      Call Relabl(ILast,Exp2%UVE,Exp2%Gam,Exp2%Oper)

C     Join together various parts of expressions Exp1 and Exp2.
C     Variables from Exp2 are marked with $
      JoinAl%Oper = Exp1%Oper * Exp2%Oper
      JoinAl%Denominator = Exp1%Denominator * Exp2%Denominator
      JoinAl%Exponent = Exp1%Exponent * Exp2%Exponent
      JoinAl%UVE = Exp1%UVE * Exp2%UVE
      JoinAl%Gam = Exp1%Gam * Exp2%Gam
      If(IPrint.ge.10) Call Output(6,JoinAl,'Before Ampers',-1)

C     Make all labels of gamma matrices to be '&'
      Call Ampers(JoinAl%Gam)
C      Call Delta_Product(Exp1%Delta,Exp2%Delta,JoinAl)

      End Function JoinAl
C############################################################
      End Module ModXpr
